#include <stddef.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "size.h"
#include "block.h"
#include "raw_chunk_manager.h"
#include "chunk_manager.h"
#include "malloc.h"

void* allocs_tree[3] = { 0, 0, 0};

void *alloc_tiny(size_t size)
{
	chunk_t chunk;

	if (allocs_tree[TINY] == NULL)
	{
		if (add_new_block(&chunk, N, NULL, false))
			return NULL;
		allocs_tree[TINY] = chunk.current;
	}
	else
	{
		if (get_append_unused_chunk(allocs_tree[TINY], size, &chunk, N))
			return NULL;
	}
	if (chunk_split(&chunk, size))
		return NULL;
	return chunk.data_start;
}

void *alloc_small(size_t size)
{
	(void)size;
	return NULL;
}

void *alloc_large(size_t size)
{
	chunk_t new_chunk;
	void    *first = allocs_tree[LARGE];

	if (first == NULL)
	{
		if (add_new_block(&new_chunk, size, NULL, true))
			return NULL;
		allocs_tree[LARGE] = new_chunk.current;
		return new_chunk.data_start;
	}

	void    *last = raw_get_last_chunk(first);
	
	if (add_new_block(&new_chunk, size, last, true))
		return NULL;
	
	return new_chunk.data_start;
}

void *ft_malloc(size_t size)
{
	if (size < n)
		return alloc_tiny(size);
	if (size < m)
		return alloc_small(size);
	return alloc_large(size); 
}